#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)
#       Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)

import sys, logging, subprocess
from cdo import Cdo
import magic
import zipfile

# Local import
from utils.utils_files import *

        

"""
Class: File format checks
---
- Checks for file format correctness
- Uses bash 'file' operation
"""
class file_format_check:

    def __init__(self, logdir, path, pattern):
        self.testid = 'test00-file-format'
        self.logdir = logdir
        self.path = path
        self.files = dir_or_file(path, pattern)

    """
    This function uses the CDO module, with command sinfon to retrieve file format
    To be used with gridded files
    """
    def cdo_return_format(self, filename):

        CDO = Cdo()
        files_format = list()
        f = lambda x: 'File format' in x
        file_format = list(filter(f, CDO.sinfon(input=filename)))[0]
        return file_format.split(':')[1].strip()

    """
    This function returns the type of a file (using the bytes: equivalent to file UNIX operation)
    To be used with other files
    """
    def file_return_format(self, filename):    
        return magic.from_file(filename)

    """
    This function returns the type of the files contained in a zip file
    For that purpose creates a temporary dir
    """
    def zip_return_formats(self, filename):

        formats = []
        with tempdir() as tmpdir:
            with zipfile.ZipFile(filename, 'r') as zipf:
                zipf.extractall(tmpdir)
                for f in os.listdir(tmpdir):
                    if f.endswith('.grib') or f.endswith('.grb') or f.endswith('.nc'):
                        formats.append(self.cdo_return_format(os.path.join(tmpdir, f)))
                    else:
                        formats.append(self.file_return_format(os.path.join(tmpdir, f)))

        return formats

    def main(self):

        # Progress init
        i = 0
        n = len(self.files)
        print()

        # Loop over files
        ffchecks = []
        for f in self.files:

            # Progress update
            i+=1
            print_progress(i, n)                                      

            if f.endswith('.grib') or f.endswith('.grb') or f.endswith('.nc'):
                ffchecks.append(self.cdo_return_format(f))
            elif zipfile.is_zipfile(f):
                ffchecks += self.zip_return_formats(f)
            else:
                ffchecks.append(self.file_return_format(f))

        # Only unique values
        ffchecks = list(set(ffchecks))

        # Gridded files check (directory contains)        
        for f in ffchecks:
            if 'grib' in f.lower() or 'netcdf' in f.lower():
                res = 'ok'  
                msg = 'File formats found: {}'.format(ffchecks)              

        # GRIB1 check        
        if 'GRIB' in ffchecks:
            res = 'err'
            msg = 'GRIB version 1 is deprecated. The recommendation is to use version 2'

        # Prepare output
        return { 
            'res': res, 
            'msg': msg
        }


if __name__ == "__main__":        
    sys.exit()
